Index: res/res_musiconhold.c
===================================================================
--- res/res_musiconhold.c	(revision 96689)
+++ res/res_musiconhold.c	(working copy)
@@ -863,23 +863,22 @@
 
 	AST_LIST_LOCK(&mohclasses);
 	if ((mohclass = get_mohbyname(moh->name, 0))) {
-		mohclass->delete = 0;
-		if (reload) {
-			ast_log(LOG_DEBUG, "Music on Hold class '%s' left alone from initial load.\n", moh->name);
-		} else {
-			ast_log(LOG_WARNING, "Music on Hold class '%s' already exists\n", moh->name);
-		}
-		free(moh);	
-		AST_LIST_UNLOCK(&mohclasses);
-		return -1;
+		if ( !mohclass->delete ) {
+			mohclass->delete = 0;
+ 			ast_log(LOG_WARNING, "Music on Hold class '%s' already exists\n", moh->name);
+			free(moh);	
+			AST_LIST_UNLOCK(&mohclasses);
+			return -1;
+ 		}
 	}
+
 	AST_LIST_UNLOCK(&mohclasses);
 
 	time(&moh->start);
 	moh->start -= respawn_time;
 	
 	if (!strcasecmp(moh->mode, "files")) {
-		if (!moh_scan_files(moh)) {
+		if (moh_scan_files(moh)<=0) {
 			ast_moh_free_class(&moh);
 			return -1;
 		}
@@ -1071,7 +1070,6 @@
 
 			/* Don't leak a class when it's already registered */
 			moh_register(class, reload);
-
 			numclasses++;
 		}
 	}
@@ -1315,7 +1313,31 @@
 
 static int unload_module(void)
 {
-	return -1;
+        int res = 0;
+        struct mohclass *class = NULL;
+
+	AST_LIST_LOCK(&mohclasses);
+	AST_LIST_TRAVERSE(&mohclasses, class, list) {
+		if (class->inuse > 0) {
+			res = -1;
+			break;
+		}
+	}
+	AST_LIST_UNLOCK(&mohclasses);
+	if (res < 0) {
+		ast_log(LOG_WARNING, "Unable to unload res_musiconhold due to active MOH channels\n");
+		return res;
+	}
+
+	ast_uninstall_music_functions();
+	ast_moh_destroy();
+	res = ast_unregister_application(app0);
+	res |= ast_unregister_application(app1);
+	res |= ast_unregister_application(app2);
+	res |= ast_unregister_application(app3);
+	res |= ast_unregister_application(app4);
+	ast_cli_unregister_multiple(cli_moh, sizeof(cli_moh) / sizeof(struct ast_cli_entry));
+	return res;
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS, "Music On Hold Resource",
